/*--------------------------------------------------------------------------
     "moleclst.h" - molecule list administration classes
        This is a part of KUCRS:
           Knowledge-basing Utilities for Complex Reaction Systems
                        (c) 2002-2010 by A. Miyoshi, University of Tokyo
                                       created: Sept. 12, 2002
                                        edited: Dec.  10, 2003 (Rev. 0.15 alpha)
                                   last edited: Feb.  20, 2010
--------------------------------------------------------------------------*/

#ifndef _MOLECLST_H_
#define _MOLECLST_H_

#include "molecule.h"
using namespace std;

/*===== 'nameControl' and related classes ================================*/

class namePool : public set<string> {
 public:                                     // --- member functions ---
  int regName(string name);
  int getNamesBeginWith(string begn, vector<string> &names);
  void print(ostream &os);
 private:
  int isUsed(string name);
};
                                                      // --- globals ---
ostream &operator<<(ostream &os, namePool &nl);

class baseNameCntrl : public map<string, int> {
 public:                                     // --- member functions ---
  void updateByName(string name);
  string updateGetAuto(string bnam, namePool &nl);
  void updateByBnamSfx(string bnam, string sfx, namePool &nl);
  void print(ostream &os);
 private:
  baseNameCntrl::iterator setNewBasename(string bnam, namePool &nl);
  string idxToSfx(int idx);
  int sfxToIdx(string sfx);
};
                                                      // --- globals ---
ostream &operator<<(ostream &os, baseNameCntrl &bl);

class nameControl {
 private:                                        // --- data members ---
  namePool nameLib;
  baseNameCntrl baseLib;
 public:                                     // --- member functions ---
  void clear();
  string regName(string name, string bnam);
  string regBaseSfx(string bnam, string sfx);
  string regBaseAuto(string bnam);
  void print(ostream &os);
};
                                                      // --- globals ---
ostream &operator<<(ostream &os, nameControl &nc);

/*===== 'molecList' and related classes ==================================*/

class molecRef {
 public:                                         // --- data members ---
  string name;
  molecInfo molec;
  int isRxnDone;
  string chemForm;
  long mtyp;
 public:                                     // --- member functions ---
  molecRef();
  molecRef(molecule &m, groupLib &gl);
  void clear();
  int restoreFrom(ifstream &ifs, nameControl &nc, groupLib &gl);
  int isRO2ofDone();
 private:
  void setFrom(molecule &m, groupLib &gl);
};

class molecList : public vector<molecRef> {
 public:                                     // --- member functions ---
  int appendFrom(string flname, libPath &libp, nameControl &nc,
   groupLib &gl);
  int findIdxOf(molecInfo &m);
  int findIdxOf(molecule &m);
  molecList::iterator find(molecInfo &m);
  molecList::iterator find(molecule &m);
  molecList::iterator find(string nam);
  void removeMol(molecInfo &m);
  void removeMol(molecule &m);
};

class molecInQue {
 public:                                         // --- data members ---
  molecule mol;
  string chemForm;
 public:                                     // --- member functions ---
  molecInQue();
  molecInQue(molecule &m);
  void clear();
  void print(ostream &os);
 private:
  void setFrom(molecule &m);
};
                                                      // --- globals ---
ostream &operator<<(ostream &os, molecInQue &miq);

class molecQueue : public deque<molecInQue> {
 public:                                     // --- member functions ---
  void appendFrom(molecQueue &mq);
  int findIdxOf(molecInfo &m);
  int findIdxOf(molecule &m);
  molecQueue::iterator find(molecInfo &m);
  molecQueue::iterator find(molecule &m);
  int getPopQueTop(molecule &m);
  void print(ostream &os);
};
                                                      // --- globals ---
ostream &operator<<(ostream &os, molecQueue &mq);

/*===== 'specAdmin' class ================================================*/

class specAdmin {
 public:                                         // --- data members ---
  nameControl namAd;
  molecList molAd;
  molecQueue rxnQue;
 public:                                     // --- member functions ---
  void clear();
  int getMolStat(molecInfo &m, string &nam);
  int getMolStat(molecule &m, string &nam);
};

#endif

