/*--------------------------------------------------------------------------
     "mollib.cc" - molecular library processor
        This is a part of KUCRS:
           Knowledge-basing Utilities for Complex Reaction Systems
                        (c) 2002-2015 by A. Miyoshi, University of Tokyo
                                       created: Aug.  23, 2002
                                        edited: Dec.  10, 2003
                                        edited: Mar.  10, 2010
                                   last edited: Dec.   4, 2015
--------------------------------------------------------------------------*/

#include "molecule.h"
using namespace std;

/*===== main routien =====================================================*/

int main(int argc, char *argv[]) {
  long mtyp;
  int setRxnDone = false, resetRxnDone = false, checkDup = false, im;
  string cdate, chemForm, clopt = "", typ;
  groupLib grp;
  typeMolec tym;
  vector<string> vs;
  molecule spc;
  vector<molecule> lstm;

  if ((argc < 3) || (argc > 4))
    { cout << "usage: mollib input output [-s|r|c]\n"; return 1; }
  if (argc == 4) {
    clopt = argv[3];
    if (clopt == "-s") { setRxnDone = true; }
    else if (clopt == "-r") { resetRxnDone = true; }
    else if (clopt == "-c") { checkDup = true; }
    else { cout << "usage: mollib input output [-s|r|c]\n"; return 1; }
  }
  cdate = "(" + strCurDate() + ")";
  cout << "--- Program mollib " << cdate << " ---\n";
  ifstream ifs(argv[1]);
  ofstream ofs(argv[2]);
  lstm.clear();
  while (spc.restoreFrom(ifs) == NORM) {
    if (checkDup) {
      spc.beUnique();
      for (im = 0; im < lstm.size(); im++) {
        if (spc.molec == lstm[im].molec) {
          cerr << "Warning: species [" << spc.molName << "] is same as ["
               << lstm[im].molName << "]\n";
        }
      }
      lstm.push_back(spc);
    } else if (!spc.isUnique) { spc.beUnique(); }
    if (setRxnDone) { spc.isRxnDone = true; }
    else if (resetRxnDone) { spc.isRxnDone = false; }
    // transform old true (-1) to new one (1)
    if (spc.isRxnDone) { spc.isRxnDone = true; }
    spc.getUniqueView(vs);
    spc.storeTo(ofs);
    chemForm = spc.molec.getChemFormula();
    cout << "-----------------------------------------------------------\n";
    cout << " name:    " << spc.molName << endl;
    cout << " formula: " << chemForm << endl;
    cout << " SMILES:  " << spc.getSMILES() << endl;
    cout << " struct:\n";
//    cout << " neqc : " << spc.numEquivChain << endl;
    cout << vs;
    cout << " groups: " << spc.getThermInput(grp) << endl;
    mtyp = spc.getType(grp); typ = tym.typNameExt(mtyp);
    cout << " code:    " << hex << mtyp << dec;
    if (typ != "") { cout << " (" << typ << ")"; }
    cout << endl;
  }
  ofs.close();
  ifs.close();
  return 0;
}

