C-----------------------------------------------------------------------
C    'thermpac.f' - process THERM .lst format file & create pac99 input
C       This is a part of KUCRS:
C          Knowledge-basing Utilities for Complex Reaction Systems
C                      (c) 2002-2016 by A. Miyoshi, University of Tokyo
C                                      created: Feb.  29, 2016
C                                       edited: Mar.   1, 2016
C                                  last edited: Apr.  18, 2016
C-----------------------------------------------------------------------
C  Constants:
C     MXLL          maximum length of input strings
C     MXLFN         maximum length of file names
C     LOUT          unit number for standard output
C     LLST          unit number for .lst input
C     LI97          unit number for .i97 output
C     MXNELM        maximum number of elements
C     NTOPN         number of temperature points for output
C     MXND          maximum number of data points
C     MXNP          maximum number of parameters
C     LWA           size of work array for LMDER1
C  Variables:
C     X()           fitting parameters
C     FVEC()        function values
C     FJAC()        Jacobian
C     WA()          work array for LMDER1
C     IPVT()        permutaion matrix
C     NATMEL()      elemental composition
C     CP()          isobaric heat capacity (input)
C     TOL           tolerance for LMDER1
C     RGAS          gas constant
C     LINE          line input
C     SPNAM         species name
C      SPN1,SPN2     and work strings
C     SDATE         date for thermodata
C     SSTATE        state (gas or condensed)
C     ELNAM()       names of elements
C     CHMF          chemical formula
C     NINTR         number of internal rotors
C     NFREE         number of internal free rotors
C     NCPD          number of Cp data points
C     ENHO          effective number of harmonic oscillator
C     T()           temperature (input)
C     CPR()         Cp/R (input)
C     TO()          temperature for output
C
      PROGRAM THRMPC
      IMPLICIT DOUBLE PRECISION(A-H,O-Z),INTEGER(I-N)
      PARAMETER (MXLL=256,MXLFN=128,LOUT=6,LLST=7,LI97=8)
      PARAMETER (MAXNELM=4,NTOPN=29)
      PARAMETER (MXND=7,MXNP=5,LWA=200)
      DIMENSION X(MXNP),FVEC(MXND),FJAC(MXND,MXNP),WA(LWA),CP(MXND)
      DIMENSION IPVT(MXNP),NATMEL(MAXNELM)
      DIMENSION TO(NTOPN)
      EXTERNAL LMDER1,FCN
      CHARACTER LINE*(MXLL),FLNLST*(MXLFN),FLNI97*(MXLFN)
      CHARACTER SPNAM*16,SPN1*10,SPN2*6,SDATE*6,CHMF*20,SSTATE*1
      CHARACTER ELNAM(MAXNELM)*2
      COMMON /THPCD/ ENHO,T(MXND),CPR(MXND)
      DATA TOL,RGAS,TREF /1D-15,8.31446D0,298.15D0/
      DATA T/300D0,400D0,500D0,600D0,800D0,1000D0,1500D0/
      DATA TO/200D0,298.15D0,300D0,400D0,500D0,600D0,700D0,800D0,900D0,
     .      1000D0,1100D0,1200D0,1300D0,1400D0,1500D0,1600D0,1800D0,
     .      2000D0,2200D0,2400D0,2600D0,2800D0,3000D0,3300D0,3600D0,
     .      3900D0,4200D0,4600D0,5000D0/
C
C      - process command-line arguments
      NARG=IARGC()
      IF (NARG.NE.2) THEN
        WRITE(LOUT,'(A)') ' usage: thermpac input.lst output.i97'
        STOP
      END IF
      CALL GETARG(1,FLNLST)
      CALL GETARG(2,FLNI97)
C
      OPEN(LLST,STATUS='OLD',FORM='FORMATTED',FILE=FLNLST)
      OPEN(LI97,STATUS='UNKNOWN',FORM='FORMATTED',FILE=FLNI97)
C      - skip first three lines
      READ(LLST,'(A)') LINE
      READ(LLST,'(A)') LINE
      READ(LLST,'(A)') LINE
   10 CONTINUE
        READ(LLST,710,END=90) SPN1,DELH,SENT,(CP(I),I=1,7),SPN2,SDATE,
     .   (ELNAM(I),NATMEL(I),I=1,4),SSTATE,NINTR
  710   FORMAT(1X,A10,2(F8.0,1X),7F8.0,1X,A6,2X,A6,3X,4(A2,I3,1X),A1,I2)
        SPNAM=SPN1
        CALL STRCAT(SPNAM,SPN2)
        NATOMS=0
        NELM=4
        DO I=4,1,-1
          IF ((ELNAM(I).EQ.'  ').OR.(NATMEL(I).LT.1)) THEN
            NELM=I-1
          ELSE
            NATOMS=NATOMS+NATMEL(I)
          END IF
        END DO
        CALL CHMFRM(CHMF,ELNAM,NATMEL,NELM)
        NCPD=7
        IF (CP(7).LE.0D0) NCPD=6
        NFREE=0
        NADDF=0
   20   CONTINUE
        ENHO=3D0*NATOMS-6D0-0.5D0*NINTR-0.5D0*NFREE+NADDF
        DO I=1,NCPD
          CPR(I)=CP(I)/RGAS-4D0-0.5D0*NFREE
        END DO
        X(1)=ENHO*0.2D0
        X(2)=ENHO*0.45D0
        X(3)=350D0
        X(4)=1700D0
        X(5)=4100D0
        CALL LMDER1(FCN,NCPD,5,X,FVEC,FJAC,MXND,TOL,INFO,IPVT,WA,LWA)
        IF ((INFO.LT.1).OR.(INFO.GT.3)) THEN
          IF (NFREE.GT.0) THEN
            NFREE=NFREE-1
            GO TO 20
          ELSE IF (NADDF.LE.5) THEN
            NADDF=NADDF+1
            GO TO 20
          ELSE
            WRITE(LOUT,610) INFO
  610       FORMAT(' Error: LMDER1 returned INFO=',I1)
            STOP 5
          END IF
        END IF
        IF ((X(3).LE.0D0).OR.(X(4).LE.0D0).OR.(X(5).LE.0D0)) THEN
          IF ((NINTR-NFREE).GT.0) THEN
            NFREE=NFREE+1
            GO TO 20
          ELSE
            NADDF=NADDF+1
            GO TO 20
          END IF
        END IF
        SUMSQ=0D0
        DO I=1,NCPD
          SUMSQ=SUMSQ+FVEC(I)*FVEC(I)
        END DO
        STDEV=SQRT(SUMSQ/NCPD)
        WRITE(LOUT,620) SPNAM(1:LENSTR(SPNAM)),STDEV,NFREE,NADDF
  620   FORMAT(' [',A,'] stdev =',F9.5,' nfree/naddf = ',I2,'/',I2)
        S0CN=SENT/RGAS-SM3V(ENHO,X,TREF)-(4D0+0.5D0*NFREE)*LOG(TREF)
C        - output to .i97 file
        WRITE(LI97,810) SPNAM
  810   FORMAT('NAME  ',A16)
        WRITE(LI97,820) CHMF,DELH
  820   FORMAT(A20,'    HF298 ',F12.4,'KJOULE')
        WRITE(LI97,830) SDATE
  830   FORMAT('DATE  ',A6)
        WRITE(LI97,840)
  840   FORMAT('LSTS  OLD'/
     .   'OUTP  MFIG              LSQS              ATM'/
     .   'METH  READIN            JOULES            ATM')
        DO I=1,NTOPN
          CPRO=CM3V(ENHO,X,TO(I))+4D0+0.5D0*NFREE
          HRTO=UM3V(ENHO,X,TO(I))+4D0+0.5D0*NFREE
          SRO=SM3V(ENHO,X,TO(I))+(4D0+0.5D0*NFREE)*LOG(TO(I))+S0CN
          WRITE(LI97,850) TO(I),CPRO,HRTO,SRO
  850     FORMAT('      T     ',F12.4,'CP/R  ',F12.4,'H-H0RT',F12.4,
     .     'S/R   ',F12.4)
        END DO
        WRITE(LI97,860)
  860   FORMAT('FINISH')
        GO TO 10
   90 CONTINUE
      CLOSE(LI97)
      CLOSE(LLST)
      STOP
      END
C
C-----------------------------------------------------------------------
C     subroutine called by LMDER
C
      SUBROUTINE FCN(M,N,X,FVEC,FJAC,LDFJAC,IFLAG)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z),INTEGER(I-N)
      PARAMETER (MXND=7)
      DIMENSION X(N),FVEC(M),FJAC(LDFJAC,N)
      COMMON /THPCD/ ENHO,T(MXND),CPR(MXND)
C
      IF (IFLAG.EQ.1) THEN
        DO I=1,M
          FVEC(I)=X(1)*F1V(X(3),T(I))+X(2)*F1V(X(4),T(I))
     .           +(ENHO-X(1)-X(2))*F1V(X(5),T(I))-CPR(I)
        END DO
      ELSE
        DO I=1,M
          FJAC(I,1)=F1V(X(3),T(I))-F1V(X(5),T(I))
          FJAC(I,2)=F1V(X(4),T(I))-F1V(X(5),T(I))
          FJAC(I,3)=X(1)*F1D(X(3),T(I))
          FJAC(I,4)=X(2)*F1D(X(4),T(I))
          FJAC(I,5)=(ENHO-X(1)-X(2))*F1D(X(5),T(I))
        END DO
      END IF
      RETURN
      END
C
C-----------------------------------------------------------------------
C     harmonic oscillator (HO) thermodynamic functions
C-----------------------------------------------------------------------
C      - heat capacity of three HO model
      DOUBLE PRECISION FUNCTION CM3V(ENHO,X,T)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z),INTEGER(I-N)
      DIMENSION X(*)
      C1=F1V(X(3),T)
      C2=F1V(X(4),T)
      C3=F1V(X(5),T)
      CM3V=X(1)*C1+X(2)*C2+(ENHO-X(1)-X(2))*C3
      END
C
C      - single HO heat capacity
      DOUBLE PRECISION FUNCTION F1V(THETA,T)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z),INTEGER(I-N)
      XV=THETA/T
      EXV=EXP(XV)
      F1V=XV*XV*EXV/(EXV-1D0)/(EXV-1D0)
      END
C
C      - single HO heat capacity derivative with respect to theta
      DOUBLE PRECISION FUNCTION F1D(THETA,T)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z),INTEGER(I-N)
      XV=THETA/T
      EXV=EXP(XV)
      F1D=XV*EXV*((2D0-XV)*(EXV+1D0)-4D0)
     .    /(EXV-1D0)/(EXV-1D0)/(EXV-1D0)/T
      END
C
C      - internal energy of three HO model
      DOUBLE PRECISION FUNCTION UM3V(ENHO,X,T)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z),INTEGER(I-N)
      DIMENSION X(*)
      U1=U1V(X(3),T)
      U2=U1V(X(4),T)
      U3=U1V(X(5),T)
      UM3V=X(1)*U1+X(2)*U2+(ENHO-X(1)-X(2))*U3
      END
C
C      - single HO internal energy
      DOUBLE PRECISION FUNCTION U1V(THETA,T)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z),INTEGER(I-N)
      XV=THETA/T
      EXV=EXP(XV)
      U1V=XV/(EXV-1D0)
      END
C
C      - entropy of three HO model
      DOUBLE PRECISION FUNCTION SM3V(ENHO,X,T)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z),INTEGER(I-N)
      DIMENSION X(*)
      S1=S1V(X(3),T)
      S2=S1V(X(4),T)
      S3=S1V(X(5),T)
      SM3V=X(1)*S1+X(2)*S2+(ENHO-X(1)-X(2))*S3
      END
C
C      - single HO entropy
      DOUBLE PRECISION FUNCTION S1V(THETA,T)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z),INTEGER(I-N)
      XV=THETA/T
      EXV=EXP(XV)
      S1V=XV/(EXV-1D0)-LOG(1D0-1D0/EXV)
      END
C
C-----------------------------------------------------------------------
C     string functions & subroutines
C-----------------------------------------------------------------------
C
C      - chemical formula
      SUBROUTINE CHMFRM(CHMF,ELNAM,NATMEL,NELM)
      CHARACTER CHMF*(*),ELNAM(*)*2,SNUM*3
      DIMENSION NATMEL(*)
      CHMF=''
      DO I=1,NELM
        CALL STRCAT(CHMF,ELNAM(I))
        WRITE(SNUM,'(I3)') NATMEL(I)
        CALL STRCAT(CHMF,SNUM)
      END DO
      END
C
C      - add string S after STRD except for spaces in S
      SUBROUTINE STRCAT(STRD,S)
      CHARACTER*(*) STRD,S
      LENSD=LEN(STRD)
      LEFS=LENSTR(S)
      LEFSD=LENSTR(STRD)
      IPOS=LEFSD
      DO I=1,LEFS
        IF (S(I:I).NE.' ') THEN
          IPOS=IPOS+1
          IF (IPOS.GT.LENSD) GOTO 100
          STRD(IPOS:IPOS)=S(I:I)
        END IF
      END DO
  100 CONTINUE
      END
C
C      - get true length of S (except for subsequent spaces)
      INTEGER FUNCTION LENSTR(S)
      CHARACTER*(*) S
      LENS=LEN(S)
      DO I=LENS,1,-1
        IF (S(I:I).NE.' ') GOTO 100
      END DO
  100 CONTINUE
      LENSTR=I
      RETURN
      END
C
